#include "gpdef.h"
#include "gpstdlib.h"
#include "gpgraphic.h"
#include "gpmm.h"
#include "gpfont.h"
#include "gpgraphic.h"
#include "gpstdio.h"

#include "stdlib.h"
#include "stdio.h"
#include "string.h"

#include "util.h"
#include "draw.h"
#include "config.h" 
#include "sound.h"


extern GP_PALETTEENTRY climber_title_Pal[256];

extern const unsigned char climber_options[5960];


int position;
int sound_onoff_position;
int save_score_position;
int sound_position;
int save_exit_position;

void WriteConfigScreen(CONFIG * config, int best_score, GPDRAWSURFACE gpDraw [2], int * nflip,  int * frame){

	GP_HPALETTE h_pal;
	//char string[256];
	int keydata;
	char string[256];
	unsigned char color;
	
	GpRectFill(NULL, &gpDraw[*nflip], 0, 0, 320, 240, 0x00);
	FlipSurface(50, gpDraw, nflip, frame);

	// Create palette
	h_pal = GpPaletteCreate(256, (GP_PALETTEENTRY*)climber_title_Pal);

	// Change palette
	GpPaletteSelect(h_pal);
	GpPaletteRealize();
		
	// Restore palette
	GpPaletteSelect(h_pal);
				
	// Release resource
	GpPaletteDelete(h_pal);
	h_pal = NULL;
	
	position = OPTION_SAVE_EXIT;
	
	sound_onoff_position = (*config).sound_onoff;
	save_score_position = (*config).save_score_onoff;
	sound_position = 1;
	save_exit_position = OPTION_SAVE;			

	GpKeyGetEx(&keydata);
	(*frame) = 0;
	while ( !(position == OPTION_SAVE_EXIT && ( keydata & KEY_START || keydata & KEY_JUMP ) && GpKeyChanged()) ) {		
		GpKeyGetEx(&keydata);

		if ( GpKeyChanged() ){
			if ( keydata & KEY_UP )
				position--;
			else if ( keydata & KEY_DOWN  )	
				position++;
			else if ( position==OPTION_SOUND_ONOFF ){
				if ( keydata & KEY_LEFT  )	
					sound_onoff_position --;
				else if ( keydata & KEY_RIGHT  )	
					sound_onoff_position ++;
				if (sound_onoff_position<0 )
					sound_onoff_position =SOUND_OFF;
				else if (sound_onoff_position>1 ) 
					sound_onoff_position =SOUND_ON;
			}	
			else if ( position==OPTION_SAVE_SCORE ){
				if ( keydata & KEY_LEFT  )	
					save_score_position --;
				else if ( keydata & KEY_RIGHT  )	
					save_score_position ++;
				if (save_score_position<0 )
					save_score_position = SAVE_SCORE_OFF;
				else if (save_score_position>1 ) 
					save_score_position = SAVE_SCORE_ON;
			}	
			else if ( position==OPTION_SOUND_TEST ){
				if ( keydata & KEY_LEFT  )	
					sound_position --;
				else if ( keydata & KEY_RIGHT  )	
					sound_position ++;
				if (sound_position<1 )
					sound_position =SOUND_TEST_MAX;
				else if (sound_position>SOUND_TEST_MAX ) 
					sound_position =1;
				if ( keydata & KEY_JUMP  )
					SoundTest(sound_position);					
				
			}	
			else if ( position==OPTION_SAVE_EXIT ){
				if ( keydata & KEY_LEFT  )	
					save_exit_position --;
				else if ( keydata & KEY_RIGHT  )	
					save_exit_position ++;
				if (save_exit_position<0 )
					save_exit_position =OPTION_EXIT;
				else if (save_exit_position>1 ) 
					save_exit_position =OPTION_SAVE;
			}				
			else if ( position	< 0 )
				position = NB_OPTIONS_MAX;
			else if ( position > NB_OPTIONS_MAX )	
				position = OPTION_SOUND_ONOFF;
			else if ( keydata & KEY_CANCEL ){
				position = OPTION_SAVE_EXIT;
				save_exit_position = OPTION_EXIT;
				keydata = KEY_START;
			}	
		}


		GpRectFill(NULL, &gpDraw[*nflip], 0, 0, 320, 240, 0x9);
		
		GpRectFill(NULL, &gpDraw[*nflip], 20, 20, 280, 200, 0x0);
		
			
			GpTransBlt(NULL, &gpDraw[*nflip],
				           320/2-149/2, 35, 
				           149, 40,
				           (unsigned char*)climber_options, 
				           0, 0, 
				           149, 40,
				           0x9);
				           
		GpTextOut(NULL, &gpDraw[*nflip], 55, 100, "SOUND", 0xFC);
		color = sound_onoff_position==SOUND_ON ? ( position==OPTION_SOUND_ONOFF ? 0x7c : 0x9 ) :  0xFC;
		GpTextOut(NULL, &gpDraw[*nflip], 175, 100, "ON", color);
		color = sound_onoff_position==SOUND_OFF ? ( position==OPTION_SOUND_ONOFF ? 0x7c : 0x9 ) :  0xFC;
		GpTextOut(NULL, &gpDraw[*nflip], 225, 100, "OFF", color);
		
		GpTextOut(NULL, &gpDraw[*nflip], 55, 125, "SAVE SCORE", 0xFC);
		color = save_score_position==SAVE_SCORE_ON ? ( position==OPTION_SAVE_SCORE ? 0x7c : 0x9 ) :  0xFC;
		GpTextOut(NULL, &gpDraw[*nflip], 175, 125, "ON", color);
		color = save_score_position==SAVE_SCORE_OFF ? ( position==OPTION_SAVE_SCORE ? 0x7c : 0x9 ) :  0xFC;
		GpTextOut(NULL, &gpDraw[*nflip], 225, 125, "OFF", color);		

		GpTextOut(NULL, &gpDraw[*nflip], 55, 155, "SOUND TEST", 0xFC);
		gp_str_func.sprintf((char*)string, sound_position<10 ? "0%d": "%d" , sound_position );
		color = position==OPTION_SOUND_TEST ? 0x7c :  0xFC;
		GpTextOut(NULL, &gpDraw[*nflip], 200, 155, string, color);

		color = position==OPTION_SAVE_EXIT && save_exit_position==OPTION_SAVE ? 0x7c :  0xFC;
		GpTextOut(NULL, &gpDraw[*nflip], 110, 185, "OK", color);
		color = position==OPTION_SAVE_EXIT && save_exit_position==OPTION_EXIT ? 0x7c :  0xFC;
		GpTextOut(NULL, &gpDraw[*nflip], 160, 185, "CANCEL", color);
				
		 
		FlipSurface(50, gpDraw, nflip, frame);	
	}
	
	if ( save_exit_position == OPTION_SAVE ){
		(*config).sound_onoff = sound_onoff_position;
		(*config).save_score_onoff = save_score_position;
		saveConfig( config, best_score );    		
	}

	GpRectFill(NULL, &gpDraw[*nflip], 0, 0, 320, 240, 0x00);
	FlipSurface(50, gpDraw, nflip, frame);	
}

void LoadFileData(char* string)
{
	unsigned long size;
	
	ERR_CODE err_code;
	F_HANDLE h_rfile;
	unsigned long n_dummy;

	err_code = GpRelativePathSet("gp:\\gpmm");
	if (err_code != SM_OK)
	{
		return;
	}

	err_code = GpFileGetSize("climber.dat", &size);
	if (err_code != SM_OK)
	{
		return;
	}

	if (size == 0)
	{
		return;
	}

	err_code = GpFileOpen("climber.dat", OPEN_R, &h_rfile);
	if (err_code != SM_OK)
	{
		return;
	}
	
	err_code = GpFileRead(h_rfile, string, size, (ulong*)&n_dummy);
	if (err_code != SM_OK)
	{
		return;
	}
	GpFileClose(h_rfile);
	
} 



void loadConfig(CONFIG * config){
	char string[256];
	char tmp[256];
	
	int pos;
	
	LoadFileData( string );
	
	
	if ( strlen(string)==0 ){
		(*config).sound_onoff = SOUND_ON;
		(*config).save_score_onoff = SAVE_SCORE_ON;	
		(*config).best_score = 0;
	}else {
		pos = strcspn(string, "|");
		strncpy(tmp, string, pos);
		(*config).best_score = atoi(tmp);
		
		strncpy(tmp, string[pos+1]+"", 1);		
		gp_str_func.sprintf((char*)tmp, "%c" , string[pos+1] );
		(*config).sound_onoff =  atoi(tmp);
		
		gp_str_func.sprintf((char*)tmp, "%c" , string[pos+3] );
		(*config).save_score_onoff = atoi(tmp);	
		
	}	
}

void SaveFileData(char * string)
{
	ERR_CODE err_code;
	F_HANDLE h_file;
	
	err_code = GpRelativePathSet("gp:\\gpmm");
	if (err_code != SM_OK)
	{
		return;
	}

	err_code = GpFileCreate("climber.dat", ALWAYS_CREATE, &h_file);
	if (err_code != SM_OK)
	{
		return;
	}

	err_code = GpFileWrite(h_file, string, strlen(string) );
	if (err_code != SM_OK)
	{
		return;
	}
	GpFileClose(h_file);
}

void saveConfig(CONFIG * config, int best_score){
	char string[256];
	
	int sound_onoff;	
	int save_score_onoff;
	int score;	
	
	score = best_score;
	sound_onoff = (*config).sound_onoff;
	save_score_onoff = (*config).save_score_onoff;
	
	
	if ( save_score_onoff==SAVE_SCORE_OFF )
		score = (*config).best_score;
	
	
	sprintf((char*)string, "%d|%d|%d" , score,sound_onoff,save_score_onoff );	
	
	SaveFileData(string);
}
